Package: satuRn
Type: Package
Title: Scalable Analysis of Differential Transcript Usage for Bulk and
        Single-Cell RNA-sequencing Applications
Date: 2022-08-05
Version: 1.10.0
Authors@R: c(person("Jeroen", "Gilis", role = c("aut","cre"),
                     email = "jeroen.gilis@ugent.be"),
             person("Kristoffer", "Vitting-Seerup", role = "ctb",
                     email = "kristoffer.vittingseerup@bio.ku.dk"),
	     person("Koen","Van den Berge", role = "ctb",
		     email = "koen.vandenberge@ugent.be"),
	     person("Lieven","Clement", role="ctb",
		     email = "Lieven.Clement@ugent.be"))
Description: satuRn provides a higly performant and scalable framework for performing
    differential transcript usage analyses. The package consists of three main functions.
    The first function, fitDTU, fits quasi-binomial generalized linear models that model
    transcript usage in different groups of interest. The second function, testDTU, tests    
    for differential usage of transcripts between groups of interest. Finally, plotDTU  
    visualizes the usage profiles of transcripts in groups of interest.
Depends: R (>= 4.1)
Imports: locfdr, SummarizedExperiment, BiocParallel, limma, pbapply,
        ggplot2, boot, Matrix, stats, methods, graphics
Suggests: knitr, rmarkdown, testthat, covr, BiocStyle, AnnotationHub,
        ensembldb, edgeR, DEXSeq, stageR, DelayedArray
VignetteBuilder: knitr
Collate: 'data.R' 'satuRn-framework.R' 'allGenerics.R' 'accessors.R'
        'fitDTU.R' 'testDTU.R' 'plotDTU.R'
License: Artistic-2.0
URL: https://github.com/statOmics/satuRn
BugReports: https://github.com/statOmics/satuRn/issues
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.2.0
biocViews: Regression, ExperimentalDesign, DifferentialExpression,
        GeneExpression, RNASeq, Sequencing, Software, SingleCell,
        Transcriptomics, MultipleComparison, Visualization
git_url: https://git.bioconductor.org/packages/satuRn
git_branch: RELEASE_3_18
git_last_commit: 97878c3
git_last_commit_date: 2023-10-24
Date/Publication: 2023-10-24
NeedsCompilation: no
Packaged: 2023-10-24 22:52:40 UTC; biocbuild
Author: Jeroen Gilis [aut, cre],
  Kristoffer Vitting-Seerup [ctb],
  Koen Van den Berge [ctb],
  Lieven Clement [ctb]
Maintainer: Jeroen Gilis <jeroen.gilis@ugent.be>
